; Fire demo for GBA
; /Mic, 2000/2003
; stabmaster_@hotmail.com | nangijala.no-ip.org
;
; I've rewritten the whole thing in ARM asm and fixed a few things while I
; was at it, so it should now run on h/w (hopefully).
;
;
; Compile with the goldroad assembler 1.7: goldroad afire.asm afire.bin
;


/* Pad the binary to 32k, for compability with newer flash carts, or whatever..  */
@fsize 32	

@textarea $8000000
@include start.asm	; boot code



/* convert 8,8,8 colors to 5:5:5 */
@macro MakeColor red,green,blue
 mov r0,red,lsr #3
 and r1,green,#$F8
 mov r1,r1,lsl #2
 orr r0,r0,r1
 and r1,blue,#$F8 
 mov r1,r1,lsl #7
 orr r0,r0,r1
@endm



/* ROM code */

start

; Copy code from ROM to IWRAM
mov r0,iwram_section
mov r1,iwram_section_ends
sub r2,r1,r0		; # of bytes to move
mov r1,#$3000000	; target address
copy_code
  ldr r3,[r0],#4
  str r3,[r1],#4
  subs r2,r2,#4
bne copy_code


mov pc,#$3000000	; branch to IWRAM


@ltorg			; dump literal pool
@endarea		; end of ROM area



iwram_section

@textarea $03000000

b iwram_entry

@include rand.asm


VSync
; In:
;  -
;
; Out:
;  -

  stmfd sp!,{r0-r1}
  mov r0,#$4000000
  add r0,r0,#4		; DISPCNT+4 = DISPSTAT

  VS_loop1
    ldrh r1,[r0]
    ands r1,r1,#1
  bne VS_loop1

  VS_loop2
    ldrh r1,[r0]
    ands r1,r1,#1
  beq VS_loop2

  ldmfd sp!,{r0-r1}
  mov pc,lr


/* Main */

iwram_entry

mov r0,#$0400
add r0,r0,#4
mov r1,#$4000000
str r0,[r1]		; set mode 4 and enable BG2

mov r3,0
mov r4,#$5000000
setup_palette
  mov r0,r3,lsl #3
  mov r1,0
  mov r2,0
  MakeColor r0,r1,r2	; r3*8, 0, 0
  strh r0,[r4]
  
  mov r0,255
  mov r1,r3,lsl #3
  mov r2,0
  MakeColor r0,r1,r2	; 255, r3*8, 0
  strh r0,[r4,#64]
  
  mov r0,255
  mov r1,255
  mov r2,r3,lsl #3
  MakeColor r0,r1,r2	; 255, 255, r3*8
  strh r0,[r4,#128]
  add r4,r4,#2
  
  add r3,r3,#1
  cmp r3,#32
bne setup_palette

mov r0,#$5000000
add r0,r0,#192
mov r1,#$8000
sub r1,r1,#1		; 7FFF = full white
mov r2,160		; 256 colors, minus the 96 already defined
set_colors_to_white
  strh r1,[r0],#2
  subs r2,r2,#1
bne set_colors_to_white


ldr r5,=484

forever

  bl VSync

  mov r6,0		; x = 2
  x_loop
  mov r7,#$6000000	; VRAM
  add r7,r7,r6
  ldr r8,=1918		; 1918 = (y * 240) + x - 2 = (8 * 240) + 0 - 2
  ldr r9,=1440		; 1440 = ((y-2) * 240) + x
  add r8,r8,r7
  add r9,r9,r7
  mov r7,8		; y = 8
  y_loop

    ldrh r0,[r8]	
    ldrh r1,[r8,#2]
    ldrh r2,[r8,#4]
    ldrh r3,[r8,r5]

    and r0,r0,#$FF	; keep the lower byte
    and r1,r1,#$FF
    and r2,r2,#$FF
    and r3,r3,#$FF

    add r0,r0,r1
    add r0,r0,r2
    add r0,r0,r3
    mov r0,r0,lsr #2	; divide the color sum by 4

    cmp r0,#0
    subgt r0,r0,#1	; subtract 1 if color average > 1
    orr r0,r0,r0,lsl #8	; copy bits 0..7 to 8..15

    strh r0,[r9]	; draw 2 pixels
    strh r0,[r9,#240]	; ..2 more

    add r8,r8,#480	; jump 2 lines down
    add r9,r9,#480

    add r7,r7,#2
    cmp r7,#160		; done ?
  ble y_loop

  bl Random16		; create a "random" number. the result is given in r0
  add r0,r0,#28		
  and r0,r0,#191

  mov r1,#$6000000
  ldr r2,=38400		; 38400 = row 240
  add r1,r1,r2
  add r1,r1,r6		; add x

  ldrh r2,[r1]		; stupid 16+ bit bus...
  and r2,r2,#$FF00
  orr r2,r2,r0
  strh r2,[r1]
  
  add r6,r6,#2
  cmp r6,#240		; done ?
blt x_loop


b forever		; loop the loop..



@ltorg			; dump literal pool
@endarea

iwram_section_ends
